@@ -60,13 +60,13 @@ func main() {
 	registerQuery(cmds, app)
 	registerRule(cmds, app)
 	registerCompact(cmds, app)
-	registerBucket(cmds, app, "bucket")
+	registerTools(cmds, app)
 	registerReceive(cmds, app)
-	registerChecks(cmds, app, "check")
+	registerQueryFrontend(cmds, app)
 
 	cmd, err := app.Parse(os.Args[1:])
 	if err != nil {
-		fmt.Fprintln(os.Stderr, errors.Wrapf(err, "Error parsing commandline arguments"))
+		fmt.Fprintln(os.Stderr, errors.Wrapf(err, "Error parsing commandline arguments: %v", os.Args))
 		app.Usage(os.Args[1:])
 		os.Exit(2)
 	}
@@ -236,7 +236,7 @@ func reload(logger log.Logger, cancel <-chan struct{}, r chan<- struct{}) error
 			level.Info(logger).Log("msg", "caught signal. Reloading.", "signal", s)
 			select {
 			case r <- struct{}{}:
-				level.Info(logger).Log("msg", "relaod dispatched.")
+				level.Info(logger).Log("msg", "reload dispatched.")
 			default:
 			}
 		case <-cancel:
@@ -244,3 +244,19 @@ func reload(logger log.Logger, cancel <-chan struct{}, r chan<- struct{}) error
 		}
 	}
 }
+
+func getFlagsMap(flags []*kingpin.FlagModel) map[string]string {
+	flagsMap := map[string]string{}
+
+	// Exclude kingpin default flags to expose only Thanos ones.
+	boilerplateFlags := kingpin.New("", "").Version("")
+
+	for _, f := range flags {
+		if boilerplateFlags.GetFlag(f.Name) != nil {
+			continue
+		}
+		flagsMap[f.Name] = f.Value.String()
+	}
+
+	return flagsMap
+}
